#!/usr/bin/env python
import pygtk
pygtk.require('2.0')

import gtk
import gnomeapplet
import sys
import os
import logging
#import locale
import gettext
from gettext import gettext as _
from optparse import OptionParser

import quitappletplus.action as actions
from quitappletplus.testing import settings as testing
from quitappletplus.pref import settings as preferencies
import quitappletplus.util as utils

PACKAGE = "quit-applet-plus"
VERSION = "1.0.1"
I18N_DOMAIN = 'quitappletplus'

tooltip = gtk.Tooltips()
NO_MASK = gtk.gdk.MOD2_MASK
CTRL_MASK = gtk.gdk.CONTROL_MASK | NO_MASK
SHIFT_MASK = gtk.gdk.SHIFT_MASK | NO_MASK
CTRL_SHIFT_MASK = gtk.gdk.CONTROL_MASK | gtk.gdk.SHIFT_MASK | NO_MASK
#ctrl+alt+mouse clashes with the compiz cube add-on:
#ALT_MASK = gtk.gdk.MOD1_MASK | NO_MASK  #MOD1_MASK = Alt key
#CTRL_ALT_MASK = gtk.gdk.CONTROL_MASK | gtk.gdk.MOD1_MASK | NO_MASK


class QuitAppletContent(object):
    
    def __init__(self, applet, iid):
        self.default_pixbuf = utils.load_pixbuf_from_file(
                                                   'quit-applet-plus_24x24.png')
        self.image = gtk.Image()
        self.image.set_from_pixbuf(self.default_pixbuf)

        self.quick_actions = {
            NO_MASK: None,
#            CTRL_ALT_MASK: None,   #see self.__rebuild_submenu()
            CTRL_SHIFT_MASK: None,   #see self.__rebuild_submenu()
            CTRL_MASK: None,       #see self.__rebuild_submenu()
        }
        self.last_mask = NO_MASK

        self.submenu = gtk.Menu()
        #self.last_show_dialog = None
        self.__rebuild_submenu() #init load icons

        self.menu = gtk.MenuItem()
        self.menu.add(self.image)
        self.menu.set_submenu(self.submenu)
        self.menu.connect("enter-notify-event", self.__mouse_entered)
        self.menu.connect("leave-notify-event", self.__mouse_leaved)
        self.menu.connect("motion-notify-event", self.__mouse_moved)

        self.menubar = gtk.MenuBar()
        self.menubar.append(self.menu)
        self.menubar.show_all()
        self.menubar.connect("button-press-event", self.__mouse_pressed)
        applet.add(self.menubar)
        self.default_tip = _("Ubuntu Quit Applet Prototype")
        tooltip.set_tip(applet, self.default_tip)

        applet.connect("change-orient", self.__on_change_orient)
        applet.connect("delete-event", self.__on_delete_event)
        applet.connect("key-press-event", self.__on_key_press)
        applet.connect("key-release-event", self.__on_key_release)
        self.applet = applet
        #gobject.timeout_add(1000, background_show, applet)

        self.propxml="""
            <popup name="button3">
                <menuitem name="Item 1" verb="Props" label="%s" pixtype="stock" pixname="gtk-properties"/>
                <menuitem name="Item 2" verb="Test" label="%s" pixtype="stock" pixname="gtk-properties"/>
                <menuitem name="Item 3" verb="About" label="%s" pixtype="stock" pixname="gnome-stock-about"/>
            </popup>
        """ % (_("_Preferences"), _("_Test scenarios"), _("_About"))
        self.verbs = [( "Props", self.__show_settings ),
                      ( "Test", self.__show_testing_settings ),
                      ( "About", self.__show_about_info ),]
        self.applet.setup_menu(self.propxml,self.verbs,None)

    def __on_key_release(self, widget, event):
        logging.debug("on_key_release: %s, %s, %s"
            % (widget, event.state, event.keyval,))
        if ((self.last_mask == CTRL_SHIFT_MASK)
                and event.keyval not in (gtk.keysyms.Control_L, gtk.keysyms.Control_R)):
            self.__update_ui(CTRL_MASK)
        else:
            self.__update_ui(NO_MASK)
#    def __on_key_release(self, widget, event):
#        logging.debug("on_key_release: %s, %s, %s"
#            % (widget, event.state, event.keyval,))
##        if (self.last_mask == CTRL_ALT_MASK
##                and event.keyval in (gtk.keysyms.Alt_L, gtk.keysyms.Alt_R)):
#        if (self.last_mask == CTRL_SHIFT_MASK
##doesn't work...                and event.keyval in (gtk.keysyms.Shift_L, gtk.keysyms.Shift_R)):
#                and event.keyval in (gtk.keysyms.ISO_Prev_Group, gtk.keysyms.ISO_Next_Group)):
#            self.__update_ui(CTRL_MASK)
#        else:
#            self.__update_ui(NO_MASK)

    def __on_key_press(self, widget, event):
        logging.debug("on_key_press: %s, %s, %s" % (widget, event.state, event.keyval))
        self.__rebuild_submenu()
        if (event.keyval in (gtk.keysyms.space, gtk.keysyms.Return)
                and (event.state == NO_MASK)):
            self.menubar.select_item(self.menu)
        elif (event.keyval in (gtk.keysyms.space, gtk.keysyms.Return)
#                and event.state == CTRL_ALT_MASK
#                and self.quick_actions[CTRL_ALT_MASK]):
                and self.last_mask == CTRL_SHIFT_MASK
                and self.quick_actions[CTRL_SHIFT_MASK]):
#            self.quick_actions[CTRL_ALT_MASK].perform_quit()
            self.quick_actions[CTRL_SHIFT_MASK].perform_quit()
            self.__update_ui(NO_MASK)
        elif (event.keyval in (gtk.keysyms.space, gtk.keysyms.Return)
                and self.last_mask == CTRL_MASK
                and self.quick_actions[CTRL_MASK]):
            self.quick_actions[CTRL_MASK].perform_quit()
            self.__update_ui(NO_MASK)
        else:
            if ((event.state == CTRL_MASK
#                 and event.keyval in (gtk.keysyms.Alt_L, gtk.keysyms.Alt_R))
                 and event.keyval in (gtk.keysyms.Shift_L, gtk.keysyms.Shift_R))
                    or
#                (event.state == ALT_MASK
                (event.state == SHIFT_MASK
                 and event.keyval in (gtk.keysyms.Control_L, gtk.keysyms.Control_R))):
#                mask = CTRL_ALT_MASK
                mask = CTRL_SHIFT_MASK
            elif (event.state == NO_MASK
                  and event.keyval in (gtk.keysyms.Control_L, gtk.keysyms.Control_R)):
                mask = CTRL_MASK
            else:
                mask = NO_MASK
            self.__update_ui(mask)
#    def __on_key_press(self, widget, event):
#        logging.debug("on_key_press: %s, %s, %s" % (widget, event.state, event.keyval))
#        self.__rebuild_submenu()
#        if (event.keyval in (gtk.keysyms.space, gtk.keysyms.Return)
#                and (event.state == NO_MASK)):
#            self.menubar.select_item(self.menu)
#        elif (event.keyval in (gtk.keysyms.space, gtk.keysyms.Return)
##                and event.state == CTRL_ALT_MASK
##                and self.quick_actions[CTRL_ALT_MASK]):
#                and event.state == CTRL_SHIFT_MASK
#                and self.quick_actions[CTRL_SHIFT_MASK]):
##            self.quick_actions[CTRL_ALT_MASK].perform_quit()
#            self.quick_actions[CTRL_SHIFT_MASK].perform_quit()
#            self.__update_ui(NO_MASK)
#        elif (event.keyval in (gtk.keysyms.space, gtk.keysyms.Return)
#                and event.state == CTRL_MASK
#                and self.quick_actions[CTRL_MASK]):
#            self.quick_actions[CTRL_MASK].perform_quit()
#            self.__update_ui(NO_MASK)
#        else:
#            if ((event.state == CTRL_MASK
##                 and event.keyval in (gtk.keysyms.Alt_L, gtk.keysyms.Alt_R))
#                 and event.keyval in (gtk.keysyms.Shift_L, gtk.keysyms.Shift_R))
#                    or
##                (event.state == ALT_MASK
#                (event.state == SHIFT_MASK
#                 and event.keyval in (gtk.keysyms.Control_L, gtk.keysyms.Control_R))):
##                mask = CTRL_ALT_MASK
#                mask = CTRL_SHIFT_MASK
#            elif (event.state == NO_MASK
#                  and event.keyval in (gtk.keysyms.Control_L, gtk.keysyms.Control_R)):
#                mask = CTRL_MASK
#            else:
#                mask = NO_MASK
#            self.__update_ui(mask)

    def __mouse_entered(self, widget, event):
        logging.debug("mouse entered")
        self.__rebuild_submenu()
        if (self.applet.flags() & gtk.HAS_FOCUS):
            return #handled by key events
        self.__mouse_moved(widget, event)

    def __mouse_moved(self, widget, event):
        #logging.debug("mouse moved")
        if (self.applet.flags() & gtk.HAS_FOCUS):
            return #handled by key events

        if (event.state == CTRL_SHIFT_MASK):
            mask = CTRL_SHIFT_MASK
#        if (event.state == CTRL_ALT_MASK):
#            mask = CTRL_ALT_MASK
        elif (event.state == CTRL_MASK):
            mask = CTRL_MASK
        else:
            mask = NO_MASK
        self.__update_ui(mask)

    def __mouse_leaved(self, widget, event):
        logging.debug("mouse leaved")
        if (self.applet.flags() & gtk.HAS_FOCUS):
            return #handled by key events
        
        self.__update_ui(NO_MASK)

    def __mouse_pressed(self, widget, event, data=None):
        logging.debug("mouse pressed: %s, %s, %s" % (event.button, event.state, event.type))
        if (event.button != 1):
            """ lets the applet context menu be shown """
            #widget.stop_emission("button-press-event")
            widget.stop_emission("button-press-event") #show context menu
        else:
            #TODO double/triple clicks should be ignored, but I have not any idea how...
            if ((event.state == CTRL_SHIFT_MASK)
                    and self.quick_actions[CTRL_SHIFT_MASK]):
#            if (event.state == CTRL_ALT_MASK
#                    and self.quick_actions[CTRL_ALT_MASK]):
#                self.quick_actions[CTRL_ALT_MASK].perform_quit()
                self.quick_actions[CTRL_SHIFT_MASK].perform_quit()
                widget.stop_emission("button-press-event")
            elif ((event.state == CTRL_MASK)
                    and self.quick_actions[CTRL_MASK]):
                self.quick_actions[CTRL_MASK].perform_quit()
                widget.stop_emission("button-press-event")
            else:
                pass
                #called by mouse enter: self.__rebuild_submenu()
        return False

    def __update_ui(self, mask):
        if (mask == self.last_mask):
            return
        new_pixbuf = self.default_pixbuf
        new_tip = self.default_tip
        action = self.quick_actions[mask]
        if (action is not None):
            new_pixbuf = action.pixbuf
            new_tip = action.info
        self.image.set_from_pixbuf(new_pixbuf)
        tooltip.set_tip(self.applet, new_tip)
        self.last_mask = mask
        logging.debug("changed to '%s' action icon, mask=%s" % (action, mask))

    def __rebuild_submenu(self):
        for item in self.submenu.get_children()[:]:
            self.submenu.remove(item)
        
        self.quick_actions[CTRL_MASK] = actions.LockScreen()
        self.submenu.append(self.quick_actions[CTRL_MASK].create_menu_item())
        if preferencies.show_switch_user:
            self.submenu.append(actions.SwitchUser().create_menu_item())
        if (testing.shutdown_permitted
                and preferencies.show_standby):
            self.submenu.append(actions.Standby().create_menu_item())
        if preferencies.show_dialog:
            quit = actions.QuitGroup()
            self.submenu.append(quit.create_menu_item())
#            self.quick_actions[CTRL_ALT_MASK] = quit
            self.quick_actions[CTRL_SHIFT_MASK] = quit
        else:
            logout = actions.LogOut()
            self.submenu.append(logout.create_menu_item())
#            self.quick_actions[CTRL_ALT_MASK] = logout
            self.quick_actions[CTRL_SHIFT_MASK] = logout
            if testing.shutdown_permitted:
                shutdown = actions.ShutDown()
                self.submenu.append(actions.Reboot().create_menu_item())
                self.submenu.append(shutdown.create_menu_item())
#                self.quick_actions[CTRL_ALT_MASK] = shutdown
                self.quick_actions[CTRL_SHIFT_MASK] = shutdown
        self.submenu.show_all()

    def __show_testing_settings(self, event, data=None):
        testing.show_settings()

    def __show_settings(self, event, data=None):
        preferencies.show_settings()

    def __show_about_info(self, event, data=None):
        about = gtk.AboutDialog()
        about.set_icon(utils.load_pixbuf_from_file('quit-applet-plus_24x24.png'))
        infos = {
            "name" : _("Quit Applet Plus"),
            "version" : VERSION,
            "authors": ["Roman Friesen https://launchpad.net/~krokosjablik",],
            "comments" : _("Ubuntu Quit Applet Prototype"),
            "copyright" : "Copyright 2008,2009 - Roman Friesen",
            "documenters" : ["Roman Friesen https://launchpad.net/~krokosjablik", ],
#            "logo-icon-name" : "gtk-quit",
            "logo" : utils.load_pixbuf_from_file("quit-applet-plus.png"),
            "website" : "https://launchpad.net/quit-applet-plus",
            "website-label" : _("Project Homepage"),
        }

        #translators: These appear in the About dialog, usual format applies.
        about.set_translator_credits( _("translator-credits") )
        
        for prop, val in infos.items():
            about.set_property(prop, val)
        
        about.connect("response", lambda about, *args: about.destroy())
        #about.set_screen(parent.get_screen())
        about.show_all()

    def __on_delete_event(self,event):
        logging.debug ('__on_delete_event: event=%s', event)
        del self.applet

    def __on_change_orient(self, arg1, data):
        logging.debug ('__on_change_orient: arg1=%s, data=%s', arg1, data)
        applet_orient = self.applet.get_orient()
        logging.debug ('applet_orient=%s' % applet_orient)
        if applet_orient == gnomeapplet.ORIENT_UP:
            menubar_pack_direction = gtk.PACK_DIRECTION_LTR
        elif applet_orient == gnomeapplet.ORIENT_DOWN:
            menubar_pack_direction = gtk.PACK_DIRECTION_RTL
        elif applet_orient == gnomeapplet.ORIENT_LEFT:
            menubar_pack_direction = gtk.PACK_DIRECTION_TTB            
        elif applet_orient == gnomeapplet.ORIENT_RIGHT:
            menubar_pack_direction = gtk.PACK_DIRECTION_TTB
        else:
            raise ValueError("applet_orient '%s' unknown!" % applet_orient)
        logging.debug ('menubar_pack_direction=%s' % menubar_pack_direction)
        self.menubar.set_pack_direction(menubar_pack_direction)


# Return a standalone window that holds the applet
def build_window():
    window = gtk.Window(gtk.WINDOW_TOPLEVEL)
    window.set_title(_("Ubuntu Quit Applet Prototype"))
    window.set_icon(utils.load_pixbuf_from_file('quit-applet-plus_24x24.png'))
    window.connect("destroy", gtk.main_quit)
    applet = gnomeapplet.Applet()
#    applet.set_flags(applet.flags() | gnomeapplet.EXPAND_MINOR)
#    applet.get_orient = lambda: gnomeapplet.ORIENT_DOWN
    applet_factory(applet, None)
    applet.reparent(window)
    return window

def applet_factory(applet, iid):
    logging.debug ('Starting Quit Applet Plus instance: %s %s', applet, iid)
    QuitAppletContent(applet, iid)
    applet.show_all()
    return True


logging.basicConfig(level=logging.DEBUG, 
    format='%(asctime)s %(name)-12s %(levelname)-8s %(message)s', 
    datefmt='%Y-%m-%d %H:%M:%S')


gettext.bindtextdomain(I18N_DOMAIN, os.path.abspath(
                        os.path.join(utils.INSTALL_DIR, 'i18n')))
if hasattr(gettext, 'bind_textdomain_codeset'):
    gettext.bind_textdomain_codeset(I18N_DOMAIN,'UTF-8')
gettext.textdomain(I18N_DOMAIN)


#locale.bindtextdomain('quitappletplus', abspath(join(deskbar.defs.DATA_DIR, "locale")))
#if hasattr(locale, 'bind_textdomain_codeset'):
#    locale.bind_textdomain_codeset('quitappletplus','UTF-8')
#locale.textdomain('quitappletplus')


# Parse commandline options
usage = "quit-applet-plus [OPTIONS]"
parser = OptionParser(usage=usage)
parser.add_option("-w", "--window", dest="window", action="store_true",
 help="Launch the applet in a standalone window for test purposes (default=no)")
parser.add_option("-d", "--dialog", dest="dialog", action="store_true",
                help="Launch only quit dialog for test purposes (default=no)")
parser.add_option("-v", "--version", dest="version", action="store_true",
                  help="Print version")
parser.add_option("--oaf-activate-iid")
parser.add_option("--oaf-ior-fd")
(options, args) = parser.parse_args()


if options.version:
    print PACKAGE + " " + VERSION
    sys.exit()

if options.window:
    import gnome
    gnome.init(PACKAGE, VERSION)
    window = build_window()
    window.show_all()
    gtk.main()
elif options.dialog:
    import gnome
    gnome.init(PACKAGE, VERSION)
    quit_group = actions.QuitGroup()
    quit_group.system_exit_on_close = True
    quit_group.perform_quit()
    gtk.main()
#    quitDialog = actions.QuitGroup().create_dialog()
#    quitDialog.connect("destroy", gtk.main_quit)
#    quitDialog.show_all()
#    gtk.main()
else:
    gnomeapplet.bonobo_factory(
            "OAFIID:Quit_Applet_Plus_Factory", 
            gnomeapplet.Applet.__gtype__, 
            PACKAGE, VERSION, 
            applet_factory)
