import os
import gtk
import logging
from gettext import gettext as _

import quitappletplus.util as utils
from quitappletplus.testing import settings as testing
from quitappletplus.pref import settings as preferencies

__author__="roman"
__date__ ="$28.11.2008 10:31:00$"


tooltip = gtk.Tooltips()


def create_label_hbox(text):
    hbox = gtk.HBox(homogeneous=False, spacing=0)
    hbox.pack_start(gtk.Label(text), expand=False)
    return hbox


def create_labeled_option(label_text, widget):
    vbox = gtk.VBox(homogeneous=False, spacing=0)
    vbox.pack_start(create_label_hbox(label_text), expand=False)
    vbox.pack_start(widget, expand=False)
    return vbox


def create_image(pixbuf):
    image = gtk.Image()
    image.set_from_pixbuf(pixbuf)
    return image


def create_menu_item(label_with_mnemonic, tip=None, pixbuf=None):
    menuitem = gtk.MenuItem()
    hbox = gtk.HBox(homogeneous=False, spacing=5)
    hbox.pack_start(create_image(pixbuf), expand=False)
    gtk_label = gtk.Label()
    gtk_label.set_text_with_mnemonic(label_with_mnemonic)
    gtk_label.set_mnemonic_widget(menuitem)
    hbox.pack_start(gtk_label, expand=False)
    menuitem.add(hbox)
    tooltip.set_tip(menuitem, tip)
    menuitem.show_all()
    return menuitem


class QuitAction(object):

    def __init__(self, label_with_mnemonic, info, icon_name, options_view=None):
        self.label_with_mnemonic = label_with_mnemonic
        self.label = label_with_mnemonic.replace('_', '')
        self.info = info
#        self.icon_name = icon_name
        self.pixbuf = utils.load_pixbuf(icon_name)
        self._options_view = options_view
        self.is_shutdown_type = False
        self.is_default = False
        self.is_standalone = True
        logging.debug("'%s' action created" % self.label)

    def get_options_view(self):
        if(self._options_view == None):
            self._options_view = gtk.Label(_("No options available."))
        return self._options_view

    options_view = property(get_options_view)

    def create_menu_item(self):
        pixbuf = self.pixbuf
        if not preferencies.show_menu_icons:
            pixbuf = None
        item = create_menu_item(self.label_with_mnemonic, self.info, pixbuf)
        item.connect_object("activate", lambda e: self.perform_quit(), None)
        return item
#    def create_menu_item(self):
#        #logging.debug("'%s': create_menu_item() called" % self.label)
#        menuitem = gtk.MenuItem()
#        hbox = gtk.HBox(homogeneous=False, spacing=5)
#        hbox.pack_start(create_image(self.pixbuf), expand=False)
#        gtk_label = gtk.Label()
#        gtk_label.set_text_with_mnemonic(self.label_with_mnemonic)
#        gtk_label.set_mnemonic_widget(menuitem)
#        hbox.pack_start(gtk_label, expand=False)
#        menuitem.add(hbox)
#        menuitem.connect_object("activate", lambda e: self.perform_quit(), None)
#        tooltip.set_tip(menuitem, self.info)
#        menuitem.show_all()
#        return menuitem

    def perform_quit(self):
        if (self.is_shutdown_type
                and self.is_standalone
                and testing.another_users_logged_in):
            group = QuitGroup()
            group.show_countdown(True)
            for quit in group.quits:
                quit.is_default = (type(quit) is type(self))
            group.perform_quit()
        else:
            #logging.debug ('QuitAction.perform_quit: %s' % (self.label,))
            dialog = gtk.MessageDialog(
                flags=gtk.DIALOG_DESTROY_WITH_PARENT | gtk.DIALOG_MODAL,
                message_format=_("'%s' performed.") % self.label)
            dialog.set_icon(
                      utils.load_pixbuf_from_file('quit-applet-plus_24x24.png'))
            dialog.format_secondary_text(_("This action is not really performed"\
                                        " - it's only a graphical prototype."))
            dialog.show_all()
        logging.debug("'%s' executed" % self.label)

    def update_button_layout(self, button):
        #logging.debug("'%s': update_button_layout() called" % self.label)
        button.set_label(self.label)
        button.set_image(create_image(self.pixbuf))
        tooltip.set_tip(button, self.info)
        return button


class LockScreen(QuitAction):

    def __init__(self):
        QuitAction.__init__(self, _("L_ock Screen"),
            _("Launch the screensaver and password protect your session"),
            "gnome-lockscreen")


class Standby(QuitAction):

    def __init__(self):
        QuitAction.__init__(self, _("Stand_by"),
                            _("Set computer inactive (power saving mode)"),
                            "gnome-session-suspend")


class SwitchUser(QuitAction):

    def __init__(self):
        QuitAction.__init__(self, _("S_witch User"),
                          _("Let another user log in or start a guest session"),
                          "gnome-session-switch")

    def create_menu_item(self):
        submenu = gtk.Menu()
        for user in testing.get_other_users():
            item = create_menu_item(user,
                                _("Lock your session and let the user log in"))
            item.connect_object("activate", lambda e: self.perform_quit(), None)
            submenu.append(item)
        submenu.append(gtk.SeparatorMenuItem())
        item = create_menu_item(_("_Guest Session"),
                     _("Lock your session and start a temporary guest session"))
        item.connect_object("activate", lambda e: self.perform_quit(), None)
        submenu.append(item)
        pixbuf = self.pixbuf
        if not preferencies.show_menu_icons:
            pixbuf = None
        menu = create_menu_item(self.label_with_mnemonic, self.info, pixbuf)
        menu.set_submenu(submenu)
        menu.show_all()
        return menu


class Reboot(QuitAction):

    def __init__(self):
        QuitAction.__init__(self, _("_Restart"),
                            _("Restart the computer"), "gnome-session-reboot")
        self.is_shutdown_type = True

        os_combo = gtk.combo_box_new_text()
        os_combo.append_text(_("Current OS (%s)") % utils.get_ubuntu_version())
        os_combo.append_text(_("Debian"))
        os_combo.append_text(_("Windows XP"))
        os_combo.append_text(_("Mac OS"))
        os_combo.set_active(0)

        self._options_view = gtk.VBox(homogeneous=False, spacing=5)
        self._options_view.set_border_width(10)
        self._options_view.pack_start(create_labeled_option(_("Restart into"), os_combo), expand=False)

class LogOut(QuitAction):

    def __init__(self):
        QuitAction.__init__(self, _("_Log Out"),
                _("Close your session and return to the login screen"),
                "gnome-session-logout")
        self.default_info = self.info
        self.default_pixbuf = self.pixbuf
        self.button = None
        self.save_session_checkbox = preferencies.create_save_session_checkbox()
        self.save_session_checkbox.connect("toggled", self.__save_session)
        self.__update_ui()

        self._options_view = gtk.VBox(homogeneous=False, spacing=10)
        self._options_view.set_border_width(10)
        self._options_view.pack_start(self.save_session_checkbox, expand=False)
        self._options_view.pack_start(create_labeled_option(
                        _("After log out switch to"), self.__create_users_cb()),
                        expand=False)

    def update_button_layout(self, button):
        self.button = super(LogOut, self).update_button_layout(button)

    def __update_ui(self):
        self.info = self.default_info
        self.pixbuf = self.default_pixbuf
        if preferencies.save_session:
            #logging.debug("LogOut.__update_ui: save_session=True")
            self.info = _("Close your session and return to the login screen (saved mode)")
            self.pixbuf = utils.load_pixbuf_from_file('logout-saved.png')

    def __save_session(self, checkbox):
        preferencies.save_session = self.save_session_checkbox.get_active()
        self.__update_ui()
        self.update_button_layout(self.button)

    def __create_users_cb(self):
        combo = gtk.combo_box_new_text()
        combo.append_text(_("---"))
        for user in testing.get_other_users():
            combo.append_text(user)
        combo.append_text(_("Guest session"))
        combo.set_active(0)
        tooltip.set_tip(combo, _("Switch to selected user after log out"))
        return combo

class ShutDown(QuitAction):

    def __init__(self):
        QuitAction.__init__(self, _("_Power Off"), _("Power off the computer"), "gnome-session-halt")
        self.default_info = self.info
        self.default_pixbuf = self.pixbuf
        self.button = None
        self.is_shutdown_type = True
        self.is_default = True
        self.__update_ui()

        self.hibernate_checkbox = preferencies.create_hibernate_checkbox()
        self.hibernate_checkbox.connect("toggled", self.__hibernate_changed)

        self._options_view = gtk.VBox(homogeneous=False, spacing=5)
        self._options_view.set_border_width(10)
        self._options_view.pack_start(self.hibernate_checkbox, expand=False)

    def update_button_layout(self, button):
        self.button = super(ShutDown, self).update_button_layout(button)

    def __update_ui(self):
        self.info = self.default_info
        self.pixbuf = self.default_pixbuf
        if preferencies.is_hibernate:
            #logging.debug("ShutDown.__update_ui: is_hibernate=True")
            self.info = _("Power off the computer (saved mode)")
            self.pixbuf = utils.load_pixbuf_from_file('power-off-saved.png')

    def __hibernate_changed(self, checkbox):
        preferencies.is_hibernate = self.hibernate_checkbox.get_active()
        self.__update_ui()
        self.update_button_layout(self.button)


class QuitGroup(QuitAction):

    def __init__(self):
        QuitAction.__init__(self, _("_Quit..."), _("Log Out, Restart or Power Off"), "gtk-quit")
        self.options_expander = None
        self.ok_button = None
        self.selected_quit = None
        self.can_shutdown = testing.shutdown_permitted
        self.users_warning_view = None
        self.dialog = None
        self.system_exit_on_close = False #True only if "./quitappletplus.py -d"
        self.quits = (
            LogOut(),
            Reboot(),
            ShutDown(),
        )
        for quit in self.quits:
            quit.is_standalone = False
            if not testing.shutdown_permitted:
                quit.is_default = not quit.is_shutdown_type
        self.__countdown_show = (preferencies.countdown_seconds > 0)
        self.__countdown = utils.Countdown(preferencies.countdown_seconds)
        if (testing.another_users_logged_in
                and testing.shutdown_permitted):
            self.__countdown.init_seconds = 300

    def show_countdown(self, show):
        assert self.dialog is None, "show_countdown() can be called only before perform_quit()!"
        self.__countdown_show = show

    def perform_quit(self):
        assert self.dialog is None, "QuitGroup.perform_quit() can be called once!"
        self.dialog = self.__create_dialog()
        self.dialog.show_all()
        if self.__countdown_show:
            default_quit = None
            for quit in self.quits:
                if quit.is_default:
                    default_quit = quit
                    break
            self.__countdown.start(default_quit)

    def __create_dialog(self):
        self.options_expander = gtk.expander_new_with_mnemonic(_("Op_tions"))
        self.options_expander.set_use_underline(True)
        self.options_expander.connect("activate", lambda x: self.__stop_countdown())
        self.ok_button = gtk.Button()
        self.ok_button.set_flags(gtk.CAN_DEFAULT)
        self.ok_button.connect("clicked", self.__quit_button_clicked)

        quit_choice_view = self._create_quit_choice_view(self.quits)

        cancel_button = gtk.Button(stock=gtk.STOCK_CANCEL)
        cancel_button.connect("clicked", self.__cancel_button_clicked)
        help_button = gtk.Button(stock=gtk.STOCK_HELP)
        help_button.connect("clicked", self.__help_button_clicked)

        dialog_title = "%s  (%s / %s)" % (self.label, os.environ['USER'],
                                    utils.get_ubuntu_version())
        dialog = gtk.Dialog(title=dialog_title, parent=None,
                        flags=gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT)
        dialog.set_icon(utils.load_pixbuf_from_file('quit-applet-plus_24x24.png'))
        if (not testing.shutdown_permitted):
            if preferencies.permission_info:
                info_view = self.__create_permission_info_view()
                dialog.vbox.pack_start(info_view)
        elif testing.another_users_logged_in:
            self.users_warning_view = self.__create_logged_users_warning()
            dialog.vbox.pack_start(self.users_warning_view)
            self.ok_button.set_sensitive(False)
            self.can_shutdown = False
        else:
            pass
        dialog.vbox.pack_start(quit_choice_view)
        if self.__countdown_show:
            dialog.vbox.pack_start(self.__countdown.get_view(), False)
        dialog.action_area.add(cancel_button)
        dialog.action_area.add(self.ok_button)
        dialog.action_area.add(help_button)
        dialog.action_area.set_child_secondary(help_button, True)
        dialog.set_default(self.ok_button)
        dialog.connect("destroy", self.__dialog_destroyed)
        if self.system_exit_on_close: #only if "./quitappletplus.py -d"
            dialog.connect("destroy", gtk.main_quit)
        return dialog

    def _create_quit_choice_view(self, quit_actions):
        assert len(quit_actions) > 0, "quit_action list is empty!"
        radioView = gtk.VBox(True, 25)
        radioView.set_border_width(0)
        defaults = None
        button = None
        for quit in quit_actions:
            button = gtk.RadioButton(button, quit.label_with_mnemonic)
            button.set_use_underline(True)
            tooltip.set_tip(button, quit.info)
            button.connect("toggled", self.__quit_changed, quit)
            if testing.shutdown_permitted:
                if quit.is_default:
                    button.set_active(True)
                    defaults = (button, quit)
            else:
                if quit.is_shutdown_type:
                    button.set_sensitive(False)
                else:
#                    button.set_active(True) #the last one wins
#                    defaults = (button, quit)
                    if quit.is_default:
                        button.set_active(True)
                        defaults = (button, quit)
            radioView.pack_start(button, True, True, 0)
            if defaults is None:
                defaults = (button, quit)
        self.__quit_changed(defaults[0], defaults[1])  #update ui

        quitView = gtk.HBox(False, 20)
        quitView.set_border_width(20)
        quitView.pack_start(self.options_expander, expand=True, fill=True, padding=0)
        quitView.pack_start(gtk.VSeparator(), expand=False)
        quitView.pack_start(radioView, expand=False)
        return quitView

    def __quit_changed(self, widget, data):
        self.__stop_countdown()
        self.selected_quit = data
        self.selected_quit.update_button_layout(self.ok_button)
        if self.options_expander.get_child() != None:
            self.options_expander.remove(self.options_expander.get_child())
        self.selected_quit.options_view.show_all()
        self.options_expander.add(self.selected_quit.options_view)
        self.__update_ok_permission()
        self.options_expander.set_label(_("Op_tions for '%s'") % self.selected_quit.label)

    def __quit_button_clicked(self, widget, data=None):
        if (self.selected_quit == None):
            raise ValueError("selected_quit not setted")
        #no dialog closing for simple testing: self.__close_dialog()
        self.__stop_countdown()
        self.selected_quit.perform_quit()

    def __cancel_button_clicked(self, widget, data=None):
        self.__close_dialog()

    def __help_button_clicked(self, widget, data=None):
        self.__stop_countdown()
        dialog = gtk.MessageDialog(parent=self.dialog,
                        flags=gtk.DIALOG_DESTROY_WITH_PARENT | gtk.DIALOG_MODAL,
                        message_format=_("Thank You! Thank You very much!!!"))
        dialog.set_icon(
                      utils.load_pixbuf_from_file('quit-applet-plus_24x24.png'))
        #dialog.format_secondary_text
        dialog.format_secondary_markup(
          _("<markup>Do You want to help?! Thank you! ;)"
            "\nIt's very simple:"
            "\n- test this prototype"
            "\n- say what you think of it"
            "\nFor the forum links and other details see:"
            "\n<span foreground='blue'>https://wiki.ubuntu.com/QuitAppletPlus</span>"
            "\n\nHave fun! :)</markup>"
            ))
        dialog.show_all()

    def __close_dialog(self):
        self.dialog.hide()
        self.dialog.destroy()

    def __dialog_destroyed(self, dialog):
        self.__stop_countdown()

    def __stop_countdown(self):
        self.__countdown.stop()

    def __proceed_anyway(self, checkbox):
        self.__stop_countdown()
        self.can_shutdown = checkbox.get_active()
        self.__update_ok_permission()

    def __show_permission_info(self, checkbox):
        self.__stop_countdown()
        preferencies.permission_info = not checkbox.get_active()

    def __update_ok_permission(self):
        self.ok_button.set_sensitive(True)
        if self.selected_quit.is_shutdown_type:
            self.ok_button.set_sensitive(self.can_shutdown)
        if self.users_warning_view:
            self.users_warning_view.set_sensitive(True)
            if not self.selected_quit.is_shutdown_type:
                self.users_warning_view.set_sensitive(False)

    def __create_logged_users_warning(self):
        return self.__create_info_view(
            icon_name="dialog-warning",
            message=_("Another users are actually logged in. They could lose data,"\
                      "\nif you power off or restart the computer."\
                      " You should log out and inform them."),
            bg_color="#ffd799",
            check_label=_("Proceed anyway (not recommended)"),
            check_callback=self.__proceed_anyway,
            check_tooltip=_("Ignore this warning and power off or restart the computer")
            )

    def __create_permission_info_view(self):
        return self.__create_info_view(
            icon_name="dialog-information",
            message=_("You don't have permissions to power off or restart the computer."),
            bg_color="#fffdb5",
            check_label=_("Don't show this message next time"),
            check_callback=self.__show_permission_info,
            check_tooltip=None
            )

    def __create_info_view(self, icon_name, message, bg_color, check_label,
                           check_callback, check_tooltip):
        image = utils.create_image(icon_name, 48)

        info_label = gtk.Label(message)
        info_label.set_line_wrap(True)
        info_label.set_alignment(0.0, 0.0)

        check_button = gtk.CheckButton(check_label)
        check_button.set_active(False)
        check_button.connect("toggled", check_callback)
        tooltip.set_tip(check_button, check_tooltip)

        info_box = gtk.VBox(homogeneous=False, spacing=10)
        info_box.set_border_width(10)
        info_box.pack_start(info_label, expand=True, fill=True, padding=0)
        info_box.pack_start(check_button, expand=False)

        view = gtk.HBox(homogeneous=False, spacing=5)
        view.set_border_width(10)
        view.pack_start(image, expand=False)
        view.pack_start(info_box, expand=True, fill=True, padding=0)

        wrapper = gtk.EventBox()
        wrapper.add(view)
        wrapper.modify_bg(state=gtk.STATE_NORMAL,
                          color=wrapper.get_colormap().alloc_color(bg_color))
        return wrapper


#class TabbedQuitGroup(QuitGroup):
#
#    def __init__(self):
#        QuitGroup.__init__(self)
#        self.label = _("Quit (Tabbed)")
#
#    def _create_quit_choice_view(self, quit_actions):
#        quitView = gtk.Notebook()
#        quitView.set_tab_pos(gtk.POS_TOP)
#        for quit in quit_actions:
#            quitView.append_page(gtk.Label(quit.info), gtk.Label(quit.label))
#        return quitView
