import logging
import gtk
from gettext import gettext as _

import quitappletplus.util as utils


__author__="roman"
__date__ ="$14.12.2008 15:02:49$"

class PrefSettings(object):

    def __init__(self):
        self.show_dialog = True
        self.show_switch_user = True
        self.show_standby = True
        self.show_menu_icons = True

        self.__is_hibernate = False
        self.hibernate_box = None

        self.__save_session = False
        self.save_session_box = None

        self.__permission_info = True
        self.permission_info_box = None

        self.close_dialog = None

        self.__countdown_seconds = 0
        self.__countdown_delays = [3, 10, 30, 60]
        if (self.__countdown_seconds != 0 #never
                and self.__countdown_seconds not in self.__countdown_delays):
            #if the init value is read from a preference file
            self.__countdown_delays.append(self.__countdown_seconds)
            self.__countdown_delays.sort()
        self.countdown_combobox = None

    def show_settings(self):
        if not self.close_dialog:
            self.close_dialog = utils.CloseDialog(
                     _("Quit Applet Plus Preferences"), self.__create_view())
            self.close_dialog.dialog.connect("destroy", self.__dialog_closed)
        else:
            self.close_dialog.present()

    def get_is_hibernate(self):
        return self.__is_hibernate

    def set_is_hibernate(self, hibernate):
        self.__is_hibernate = hibernate
        if self.hibernate_box:
            self.hibernate_box.set_active(self.__is_hibernate)
        logging.debug("Pref: __is_hibernate=%s" % self.__is_hibernate)

    is_hibernate = property(fget=get_is_hibernate, fset=set_is_hibernate)

    def get_save_session(self):
        return self.__save_session

    def set_save_session(self, save_session):
        self.__save_session = save_session
        if self.save_session_box:
            self.save_session_box.set_active(self.__save_session)
        logging.debug("Pref: __save_session=%s" % self.__save_session)

    save_session = property(fget=get_save_session, fset=set_save_session)

    def get_permission_info(self):
        return self.__permission_info

    def set_permission_info(self, permission_info):
        self.__permission_info = permission_info
        if self.permission_info_box:
            self.permission_info_box.set_active(self.__permission_info)
        logging.debug("Pref: __permission_info=%s" % self.__permission_info)

    permission_info = property(fget=get_permission_info, fset=set_permission_info)

    def get_countdown_seconds(self):
        return self.__countdown_seconds

    countdown_seconds = property(fget=get_countdown_seconds)

    def __dialog_closed(self, widget):
        self.close_dialog = None

    def __change_hibernate(self, togglebutton):
        self.__is_hibernate = togglebutton.get_active()
        logging.debug("Pref: __is_hibernate=%s" % self.__is_hibernate)

    def __change_save_session(self, togglebutton):
        self.__save_session = togglebutton.get_active()
        logging.debug("Pref: __save_session=%s" % self.__save_session)

    def __change_permission_info(self, togglebutton):
        self.__permission_info = togglebutton.get_active()
        logging.debug("Pref: __permission_info=%s" % self.__permission_info)

    def __change_show_dialog(self, togglebutton):
        self.show_dialog = togglebutton.get_active()
        self.countdown_combobox.set_sensitive(self.show_dialog)
        logging.debug("Pref: show_dialog=%s" % self.show_dialog)

    def __change_show_switch_user(self, togglebutton):
        self.show_switch_user = togglebutton.get_active()
        logging.debug("Pref: show_switch_user=%s" % self.show_switch_user)

    def __change_show_standby(self, togglebutton):
        self.show_standby = togglebutton.get_active()
        logging.debug("Pref: show_standby=%s" % self.show_standby)

    def __change_show_menu_icons(self, togglebutton):
        self.show_menu_icons = togglebutton.get_active()
        logging.debug("Pref: show_menu_icons=%s" % self.show_menu_icons)

    def create_save_session_checkbox(self):
        checkbox = gtk.CheckButton(_("Save session"))
        checkbox.set_active(self.__save_session)
        checkbox.set_tooltip_text(_("Restore the actually running applications"\
                                    " on the next log in"))
        return checkbox

    def create_hibernate_checkbox(self):
        checkbox = gtk.CheckButton(_("Save computer state"))
        checkbox.set_active(self.__is_hibernate)
        checkbox.set_tooltip_text(_("Restore the current computer state"\
                                    " on the next start (Hibernation)"))
        return checkbox

    def create_permission_info_checkbox(self):
        checkbox = gtk.CheckButton(_("Show permission messages"))
        checkbox.set_active(self.__permission_info)
        checkbox.set_tooltip_text(_("Show a message if you don't have the"\
                                    " permission to power off the computer"))
        return checkbox

    def __create_show_dialog_checkbox(self):
        checkbox = gtk.CheckButton(_("Bundle 'Log Out', 'Restart' and 'Power Off' to the 'Quit...' action"))
        checkbox.set_active(self.show_dialog)
        checkbox.set_tooltip_text(_("Show 'Log Out', 'Restart' and 'Power Off'"\
                                    " actions in the quit dialog"))
        return checkbox

    def __create_show_switch_user_checkbox(self):
        checkbox = gtk.CheckButton(_("Show '%s'") % _("Switch User"))
        checkbox.set_active(self.show_switch_user)
        checkbox.set_tooltip_text(
             _("Include the '%s' action into the quit menu") % _("Switch User"))
        return checkbox

    def __create_show_standby_checkbox(self):
        checkbox = gtk.CheckButton(_("Show '%s'") % _("Standby"))
        checkbox.set_active(self.show_standby)
        checkbox.set_tooltip_text(
                 _("Include the '%s' action into the quit menu") % _("Standby"))
        return checkbox

    def __create_show_menu_icons_checkbox(self):
        checkbox = gtk.CheckButton(_("Show menu icons"))
        checkbox.set_active(self.show_menu_icons)
        checkbox.set_tooltip_text(
                 _("Show icons in the quit menu"))
        return checkbox

# no idea how to show "never" for 0 seconds
#    def __create_countdown_range(self):
#        scale = gtk.HScale(gtk.Adjustment(
#                                value=self.__countdown_seconds,
#                                lower=0,
#                                upper=60,
#                                step_incr=3,
#                                page_incr=10))
#        scale.set_digits(0)
#        scale.set_value_pos(gtk.POS_BOTTOM)
#        scale.set_update_policy(gtk.UPDATE_DISCONTINUOUS)
#        return scale

    def __countdown_seconds_changed(self, combobox):
        self.__countdown_seconds = 0
        if (combobox.get_active() > 0):
            self.__countdown_seconds = self.__countdown_delays[
                                                    combobox.get_active() - 1]
        logging.debug("Pref: countdown_seconds=%s" % self.__countdown_seconds)
#    def __countdown_seconds_changed(self, combobox):
#        if (combobox.get_active() <= 0):
#            self.__countdown_seconds = 0
#        else:
#            text = combobox.get_active_text().strip()
# BUG here if translated order is different!            self.__countdown_seconds = int(text.split(" ")[1].strip())
#        logging.debug("Pref: countdown_seconds=%s" % self.__countdown_seconds)

    def __create_countdown_combo(self):
        combo = gtk.combo_box_new_text()
        combo.append_text(_("never"))
        for delay in self.__countdown_delays:
            combo.append_text(_("after %s seconds") % delay)
        if(self.__countdown_seconds == 0):
            combo.set_active(0) #never
        else:
            index = self.__countdown_delays.index(self.__countdown_seconds)
            combo.set_active(1 + index)

        combo.set_tooltip_text(
           _("Perform the default action automatically after a delay or never"))
        return combo

    def __create_view(self):
        show_dialog_checkbox = self.__create_show_dialog_checkbox()
        show_dialog_checkbox.connect("toggled", self.__change_show_dialog)

        self.countdown_combobox = self.__create_countdown_combo()
        self.countdown_combobox.set_sensitive(self.show_dialog)
        self.countdown_combobox.connect("changed", self.__countdown_seconds_changed)
        countdown_hbox = gtk.HBox(homogeneous=False, spacing=5)
        countdown_hbox.pack_start(gtk.Label(""), expand=False)
        countdown_hbox.pack_start(
           gtk.Label(_("Perform the quit dialog default action automatically")),
           expand=False)
        countdown_hbox.pack_start(self.countdown_combobox, expand=True, fill=True, padding=0)

        show_switch_user_checkbox = self.__create_show_switch_user_checkbox()
        show_switch_user_checkbox.connect("toggled", self.__change_show_switch_user)

        show_standby_checkbox = self.__create_show_standby_checkbox()
        show_standby_checkbox.connect("toggled", self.__change_show_standby)

        show_menu_icons_checkbox = self.__create_show_menu_icons_checkbox()
        show_menu_icons_checkbox.connect("toggled", self.__change_show_menu_icons)

        self.save_session_box = self.create_save_session_checkbox()
        self.save_session_box.connect("toggled", self.__change_save_session)

        self.hibernate_box = self.create_hibernate_checkbox()
        self.hibernate_box.connect("toggled", self.__change_hibernate)

        self.permission_info_box = self.create_permission_info_checkbox()
        self.permission_info_box.connect("toggled", self.__change_permission_info)

        view = gtk.VBox(homogeneous=False, spacing=5)
        view.set_border_width(20)
        view.pack_start(show_dialog_checkbox, expand=False)
        view.pack_start(countdown_hbox, expand=False)
        view.pack_start(show_switch_user_checkbox, expand=False)
        view.pack_start(show_standby_checkbox, expand=False)
        view.pack_start(show_menu_icons_checkbox, expand=False)
        view.pack_start(gtk.Label(" "), expand=False)
        view.pack_start(self.save_session_box, expand=False)
        view.pack_start(self.hibernate_box, expand=False)
        view.pack_start(self.permission_info_box, expand=False)
        return view


settings = PrefSettings()


if __name__ == "__main__":
    print "Hello! I'm a pref modul!";