import os
import gobject
import logging
import gtk
from gettext import gettext as _
from gettext import ngettext

__author__="roman"
__date__ ="$28.11.2008 10:31:40$"

INSTALL_DIR = "/opt/quit-applet-plus/"
if (os.path.exists("../debian") and os.path.exists("../MANIFEST.in")):
    INSTALL_DIR = ".." #development

pixbuf_cache = {}
ubuntu_version = None
#tooltip = gtk.Tooltips()


def load_pixbuf(icon_name, icon_size=22):
    key = "icon_theme:%s_%s" % (icon_name, icon_size)
    if(not pixbuf_cache.has_key(key)):
        icon_theme = gtk.icon_theme_get_default()
        pixbuf_cache[key] = icon_theme.load_icon(icon_name, icon_size, 0)
        logging.debug("icon loaded: %s", key)
    return pixbuf_cache[key]


def load_pixbuf_from_file(icon_file_name):
    icon_path = os.path.join(INSTALL_DIR, 'icons', icon_file_name)
    if(not pixbuf_cache.has_key(icon_path)):
        pixbuf_cache[icon_path] = gtk.gdk.pixbuf_new_from_file(icon_path)
        logging.debug("icon loaded: %s", icon_path)
    return pixbuf_cache[icon_path]


def create_image(icon_name, icon_size):
    try:
        image = gtk.Image()
        image.set_from_pixbuf(load_pixbuf(icon_name, icon_size))
        return image
    except gobject.GError:
        logging.debug("create_image: cannot load named icon '%s'"
                        % icon_name, exc_info=True)
        raise


def get_ubuntu_version():
    global ubuntu_version
    if ubuntu_version:
        return ubuntu_version

    command = 'lsb_release -d'
    try:
        output = os.popen(command)
        line = output.readline() #for example: "Description:	Ubuntu 8.04.1"
        logging.debug("'%s' ouput: %s" % (command, line))
        output.close()
        ubuntu_version = line.split(":")[1].strip()
    except Exception:
        logging.warning("read ubuntu version '%s' failed",
                        command, exc_info=True)
        ubuntu_version = "???"
    return ubuntu_version


class Countdown(object):

    def __init__(self, init_seconds):
        """
        init_seconds can have any value here, but it must be > 0 before the
        start() method is called. So the self.init_seconds attribute must be
        adjusted according to this condition, before the start() is called.
        That allows you always to create the Countdown instance but to start it
        only if you need (if init_seconds > 0).
        """
        self.__init_seconds = init_seconds
        self.__remain_seconds = self.__init_seconds
        self.__started = False
        self.__stopped = False
        self.__completed = False
        self.__timer = None
        self.__hide_timer = None

        self.__progressbar = gtk.ProgressBar()
        self.__progressbar.set_fraction(1.0)
        self.__progressbar.set_text(" ") #see start()
        self.__progressbar.set_tooltip_text(_("Click to stop the countdown"))
        self.__progressbar_wrapper = gtk.EventBox()
        self.__progressbar_wrapper.add(self.__progressbar)
        self.__progressbar_wrapper.connect("button-press-event",
                                            self.__countdownbar_clicked)
#        self.__stop_button = gtk.Button()
#        self.__stop_button.set_image(create_image("stop", 16))
#        tooltip.set_tip(self.__stop_button, _("Stop countdown"))
#        self.__stop_button.connect("clicked", lambda x: self.stop())
        self.__view = gtk.HBox(False, 0)
        self.__view.pack_start(self.__progressbar_wrapper,
                               expand=True, fill=True, padding=0)
#        self.__view.pack_start(self.__stop_button, expand=False)

    def __countdownbar_clicked(self, widget, event):
        if (self.__started and not self.__completed):
            self.stop()

    def get_init_seconds(self):
        return self.__init_seconds

    def set_init_seconds(self, init_seconds):
        assert not self.__started, "Countdown was yet started"\
               " (init_seconds can be changed only before the countdown start)!"
        assert (init_seconds > 0), "init_seconds must be > 0!"
        self.__init_seconds = init_seconds
        self.__remain_seconds = self.__init_seconds

    init_seconds = property(fget=get_init_seconds, fset=set_init_seconds)

    def get_view(self):
        return self.__view

    def start(self, quit_action):
        assert (self.__init_seconds > 0), "init_seconds must be > 0!"
        assert not self.__started, "Countdown was yet started (can be started only once)!"
        assert quit_action, "quit_action is None!"
        self.__update_text(quit_action, self.__init_seconds)
        self.__timer = gobject.timeout_add(1000, self.__countdown, quit_action)
        self.__started = True

    def stop(self):
        if (not self.__started
                or self.__stopped
                or self.__completed):
            return
        self.__stopped = True
        gobject.source_remove(self.__timer)
        self.__timer = 0
        self.__progressbar.set_fraction(1.0)
        self.__progressbar.set_text(_("Countdown stopped"))
#        self.__stop_button.set_sensitive(False)
        self.__progressbar.set_tooltip_text(None)
        self.__hide_timer = gobject.timeout_add(3000, self.__hide_progressbar)

    def __hide_progressbar(self):
        if self.__view.get_parent(): #if not destroyed
            self.__view.remove(self.__progressbar_wrapper)
#            self.__view.remove(self.__stop_button)
        gobject.source_remove(self.__hide_timer)
        self.__hide_timer = 0
        return False

    def __countdown(self, quit_action):
        self.__remain_seconds -= 1
        new_val = (1.0/self.__init_seconds) * self.__remain_seconds
        self.__progressbar.set_fraction(new_val)
        self.__update_text(quit_action, self.__remain_seconds)
        if self.__stopped:
            return False
        if self.__remain_seconds > 0:
            return True
#        self.__stop_button.set_sensitive(False)
        self.__progressbar.set_tooltip_text(None)
        quit_action.perform_quit()
        self.__progressbar.set_text(_("'%s' performed") % quit_action.label)
        self.__completed = True
        return False

    def __update_text(self, quit_action, seconds):
        self.__progressbar.set_text(
            ngettext("'%(action)s' automatically after %(time)s second",
                   "'%(action)s' automatically after %(time)s seconds", seconds)
            % {"action":quit_action.label, "time":seconds})


class CloseDialog(object):

    def __init__(self, title, content):
        self.title = title
        self.content = content
        self.dialog = self.__create_dialog(title, content)
        self.dialog.show_all()

    def present(self):
        self.dialog.present()
#        self.deiconify()
#        self.close_dialog.set_urgency_hint(True)

    def __close_button_clicked(self, widget, data=None):
        self.dialog.hide()
        self.dialog.destroy()

    def __create_dialog(self, title, content):
        close_button = gtk.Button(stock=gtk.STOCK_CLOSE)
        close_button.set_flags(gtk.CAN_DEFAULT)
        close_button.connect("clicked", self.__close_button_clicked)

        self.dialog = gtk.Dialog(title=title, parent=None,
                                 flags=gtk.DIALOG_DESTROY_WITH_PARENT)
#                      flags=gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT)
        self.dialog.set_icon(load_pixbuf_from_file('quit-applet-plus_24x24.png'))
        self.dialog.vbox.pack_start(content)
        self.dialog.action_area.add(close_button)
        self.dialog.set_default(close_button)
        return self.dialog


#class OkApplyDialog(object):
#
#    def __init__(self, content):
#        assert hasattr(content, "get_view"), "content %s doesn't have get_view() method!" % content
#        assert hasattr(content, "save"), "content %s doesn't have save() method!" % content
#        self.content = content
#        self.dialog = self.__create_dialog()
#        self.dialog.show_all()
#
#    def __ok_button_clicked(self, widget, data=None):
#        self.content.save()
#        self.dialog.hide()
#        self.dialog.destroy()
#
#    def __apply_button_clicked(self, widget, data=None):
#        self.content.save()
#
#    def __create_dialog(self):
#        ok_button = gtk.Button(stock=gtk.STOCK_OK)
#        ok_button.set_flags(gtk.CAN_DEFAULT)
#        ok_button.connect("clicked", self.__ok_button_clicked)
#
#        apply_button = gtk.Button(stock=gtk.STOCK_APPLY)
#        apply_button.connect("clicked", self.__apply_button_clicked)
#
#        self.dialog = gtk.Dialog(title=_("Quit Applet Plus Test Environment"), parent=None,
#                                 flags=gtk.DIALOG_DESTROY_WITH_PARENT)
##                            flags=gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT)
#        self.dialog.vbox.pack_start(self.content.get_view())
#        self.dialog.action_area.add(apply_button)
#        self.dialog.action_area.add(ok_button)
#        self.dialog.set_default(ok_button)
#        return self.dialog


#def catch_exc(func):
#    def wrapper(*args, **kwargs):
#        try:
#            func(*args, **kwargs)
#        except Exception:
#            logging.error("'%s' failed", func, exc_info=True)
#    return wrapper
#@catch_exc
#def decorated(value, some=3):
#    print "value=%s, some=%s" % (value, some)
#    value = 1 / 0
#    print value
#decorated(2)